@@ -15,6 +15,7 @@ import (
 	"go.uber.org/zap"
 
 	"github.com/helmfile/helmfile/pkg/argparser"
+	"github.com/helmfile/helmfile/pkg/cluster"
 	"github.com/helmfile/helmfile/pkg/envvar"
 	"github.com/helmfile/helmfile/pkg/filesystem"
 	"github.com/helmfile/helmfile/pkg/helmexec"
@@ -28,12 +29,15 @@ var Cancel goContext.CancelFunc
 
 // App is the main application object.
 type App struct {
-	OverrideKubeContext        string
-	OverrideHelmBinary         string
-	OverrideKustomizeBinary    string
-	EnableLiveOutput           bool
-	StripArgsValuesOnExitError bool
-	DisableForceUpdate         bool
+	OverrideKubeContext             string
+	OverrideHelmBinary              string
+	OverrideKustomizeBinary         string
+	EnableLiveOutput                bool
+	StripArgsValuesOnExitError      bool
+	DisableForceUpdate              bool
+	EnforcePluginVerification       bool
+	HelmOCIPlainHTTP                bool
+	DisableKubeVersionAutoDetection bool
 
 	Logger      *zap.SugaredLogger
 	Kubeconfig  string
@@ -80,6 +84,8 @@ func New(conf ConfigProvider) *App {
 		EnableLiveOutput:           conf.EnableLiveOutput(),
 		StripArgsValuesOnExitError: conf.StripArgsValuesOnExitError(),
 		DisableForceUpdate:         conf.DisableForceUpdate(),
+		EnforcePluginVerification:  conf.EnforcePluginVerification(),
+		HelmOCIPlainHTTP:           conf.HelmOCIPlainHTTP(),
 		Logger:                     conf.Logger(),
 		Kubeconfig:                 conf.Kubeconfig(),
 		Env:                        conf.Env(),
@@ -236,6 +242,7 @@ func (a *App) Template(c TemplateConfigProvider) error {
 			Set:                    c.Set(),
 			Values:                 c.Values(),
 			KubeVersion:            c.KubeVersion(),
+			HelmOCIPlainHTTP:       a.HelmOCIPlainHTTP,
 		}, func() {
 			ok, errs = a.template(run, c)
 		})
@@ -586,7 +593,7 @@ func (a *App) dag(r *Run) error {
 }
 
 func (a *App) ListReleases(c ListConfigProvider) error {
-	var releases []*HelmRelease
+	releasesChan := make(chan []*HelmRelease, 100)
 
 	err := a.ForEachState(func(run *Run) (_ bool, errs []error) {
 		var stateReleases []*HelmRelease
@@ -612,15 +619,33 @@ func (a *App) ListReleases(c ListConfigProvider) error {
 			errs = append(errs, err)
 		}
 
-		releases = append(releases, stateReleases...)
+		if len(stateReleases) > 0 {
+			releasesChan <- stateReleases
+		}
 
 		return
 	}, false, SetFilter(true))
 
+	close(releasesChan)
+
+	// Collect all releases from channel
+	var releases []*HelmRelease
+	for rels := range releasesChan {
+		releases = append(releases, rels...)
+	}
+
 	if err != nil {
 		return err
 	}
 
+	// Sort releases to ensure deterministic output order regardless of parallel execution
+	sort.Slice(releases, func(i, j int) bool {
+		if releases[i].Namespace != releases[j].Namespace {
+			return releases[i].Namespace < releases[j].Namespace
+		}
+		return releases[i].Name < releases[j].Name
+	})
+
 	if c.Output() == "json" {
 		err = FormatAsJson(releases)
 	} else {
@@ -743,6 +768,10 @@ func (a *App) visitStateFiles(fileOrDir string, opts LoadOpts, do func(string, s
 }
 
 func (a *App) loadDesiredStateFromYaml(file string, opts ...LoadOpts) (*state.HelmState, error) {
+	return a.loadDesiredStateFromYamlWithBaseDir(file, "", opts...)
+}
+
+func (a *App) loadDesiredStateFromYamlWithBaseDir(file string, baseDir string, opts ...LoadOpts) (*state.HelmState, error) {
 	var op LoadOpts
 	if len(opts) > 0 {
 		op = opts[0]
@@ -755,6 +784,7 @@ func (a *App) loadDesiredStateFromYaml(file string, opts ...LoadOpts) (*state.He
 		chart:     a.Chart,
 		logger:    a.Logger,
 		remote:    a.remote,
+		baseDir:   baseDir,
 
 		overrideKubeContext:     a.OverrideKubeContext,
 		overrideHelmBinary:      a.OverrideHelmBinary,
@@ -802,7 +832,12 @@ func (a *App) getHelm(st *state.HelmState) (helmexec.Interface, error) {
 	key := createHelmKey(bin, kubectx)
 
 	if _, ok := a.helms[key]; !ok {
-		exec, err := helmexec.New(bin, helmexec.HelmExecOptions{EnableLiveOutput: a.EnableLiveOutput, DisableForceUpdate: a.DisableForceUpdate}, a.Logger, kubeconfig, kubectx, &helmexec.ShellRunner{
+		exec, err := helmexec.New(bin, helmexec.HelmExecOptions{
+			EnableLiveOutput:          a.EnableLiveOutput,
+			DisableForceUpdate:        a.DisableForceUpdate,
+			EnforcePluginVerification: a.EnforcePluginVerification,
+			HelmOCIPlainHTTP:          a.HelmOCIPlainHTTP,
+		}, a.Logger, kubeconfig, kubectx, &helmexec.ShellRunner{
 			Logger:                     a.Logger,
 			Ctx:                        a.ctx,
 			StripArgsValuesOnExitError: a.StripArgsValuesOnExitError,
@@ -817,117 +852,256 @@ func (a *App) getHelm(st *state.HelmState) (helmexec.Interface, error) {
 }
 
 func (a *App) visitStates(fileOrDir string, defOpts LoadOpts, converge func(*state.HelmState) (bool, []error)) error {
-	noMatchInHelmfiles := true
+	return a.visitStatesWithContext(fileOrDir, defOpts, converge, nil)
+}
 
-	err := a.visitStateFiles(fileOrDir, defOpts, func(f, d string) (retErr error) {
-		opts := defOpts.DeepCopy()
+func (a *App) processStateFileParallel(relPath string, defOpts LoadOpts, converge func(*state.HelmState) (bool, []error), sharedCtx *Context, errChan chan error, matchChan chan bool) {
+	var file string
+	var dir string
+	if a.fs.DirectoryExistsAt(relPath) {
+		file = relPath
+		dir = relPath
+	} else {
+		file = filepath.Base(relPath)
+		dir = filepath.Dir(relPath)
+	}
 
-		if opts.CalleePath == "" {
-			opts.CalleePath = f
+	absd, errAbsDir := a.fs.Abs(dir)
+	if errAbsDir != nil {
+		errChan <- errAbsDir
+		return
+	}
+
+	opts := defOpts.DeepCopy()
+	if opts.CalleePath == "" {
+		opts.CalleePath = file
+	}
+
+	st, err := a.loadDesiredStateFromYamlWithBaseDir(file, absd, opts)
+	if err != nil {
+		switch stateLoadErr := err.(type) {
+		case *state.StateLoadError:
+			switch stateLoadErr.Cause.(type) {
+			case *state.UndefinedEnvError:
+				return
+			default:
+				errChan <- appError(fmt.Sprintf("in %s/%s", dir, file), err)
+				return
+			}
+		default:
+			errChan <- appError(fmt.Sprintf("in %s/%s", dir, file), err)
+			return
+		}
+	}
+
+	if st == nil {
+		return
+	}
+
+	st.Selectors = opts.Selectors
+
+	if len(st.Helmfiles) > 0 && !opts.Reverse {
+		if err := a.processNestedHelmfiles(st, absd, file, defOpts, opts, converge, sharedCtx); err != nil {
+			errChan <- err
+			return
+		}
+	}
+
+	templated, err := st.ExecuteTemplates()
+	if err != nil {
+		errChan <- appError(fmt.Sprintf("in %s/%s: failed executing release templates in \"%s\"", dir, file, file), err)
+		return
+	}
+
+	var errs []error
+	CleanWaitGroup.Add(1)
+	var cleanErr error
+	defer func() {
+		defer CleanWaitGroup.Done()
+		cleanErr = context{app: a, st: templated, retainValues: defOpts.RetainValuesFiles}.clean(errs)
+	}()
+
+	processed, errs := converge(templated)
+
+	if len(errs) > 0 {
+		errChan <- errs[0]
+		return
+	}
+	if cleanErr != nil {
+		errChan <- cleanErr
+		return
+	}
+
+	// Report if this file had matching releases
+	if processed {
+		matchChan <- true
+	}
+
+	if opts.Reverse && len(st.Helmfiles) > 0 {
+		if err := a.processNestedHelmfiles(st, absd, file, defOpts, opts, converge, sharedCtx); err != nil {
+			errChan <- err
+		}
+	}
+}
+
+func (a *App) processNestedHelmfiles(st *state.HelmState, absd, file string, defOpts, opts LoadOpts, converge func(*state.HelmState) (bool, []error), sharedCtx *Context) error {
+	for i, m := range st.Helmfiles {
+		optsForNestedState := LoadOpts{
+			CalleePath:        filepath.Join(absd, file),
+			Environment:       m.Environment,
+			Reverse:           defOpts.Reverse,
+			RetainValuesFiles: defOpts.RetainValuesFiles,
+		}
+		if (m.Selectors == nil && !isExplicitSelectorInheritanceEnabled()) || m.SelectorsInherited {
+			optsForNestedState.Selectors = opts.Selectors
+		} else {
+			optsForNestedState.Selectors = m.Selectors
 		}
 
-		st, err := a.loadDesiredStateFromYaml(f, opts)
+		if err := a.visitStatesWithContext(m.Path, optsForNestedState, converge, sharedCtx); err != nil {
+			switch err.(type) {
+			case *NoMatchingHelmfileError:
+			default:
+				return appError(fmt.Sprintf("in .helmfiles[%d]", i), err)
+			}
+		}
+	}
+	return nil
+}
 
-		ctx := context{app: a, st: st, retainValues: defOpts.RetainValuesFiles}
+func (a *App) visitStatesWithContext(fileOrDir string, defOpts LoadOpts, converge func(*state.HelmState) (bool, []error), sharedCtx *Context) error {
+	noMatchInHelmfiles := true
 
-		if err != nil {
-			switch stateLoadErr := err.(type) {
-			// Addresses https://github.com/roboll/helmfile/issues/279
-			case *state.StateLoadError:
-				switch stateLoadErr.Cause.(type) {
-				case *state.UndefinedEnvError:
-					return nil
+	desiredStateFiles, err := a.findDesiredStateFiles(fileOrDir, defOpts)
+
+	if len(desiredStateFiles) > 1 {
+		var wg sync.WaitGroup
+		errChan := make(chan error, len(desiredStateFiles))
+		matchChan := make(chan bool, len(desiredStateFiles))
+
+		for _, relPath := range desiredStateFiles {
+			wg.Add(1)
+			go func(relPath string) {
+				defer wg.Done()
+				a.processStateFileParallel(relPath, defOpts, converge, sharedCtx, errChan, matchChan)
+			}(relPath)
+		}
+
+		wg.Wait()
+		close(errChan)
+		close(matchChan)
+
+		for err := range errChan {
+			if err != nil {
+				return err
+			}
+		}
+
+		// Check if any files had matching releases
+		for range matchChan {
+			noMatchInHelmfiles = false
+		}
+	} else {
+		// Sequential processing for single file
+		err = a.visitStateFiles(fileOrDir, defOpts, func(f, d string) (retErr error) {
+			opts := defOpts.DeepCopy()
+
+			if opts.CalleePath == "" {
+				opts.CalleePath = f
+			}
+
+			st, err := a.loadDesiredStateFromYaml(f, opts)
+
+			ctx := context{app: a, st: st, retainValues: defOpts.RetainValuesFiles}
+
+			if err != nil {
+				switch stateLoadErr := err.(type) {
+				case *state.StateLoadError:
+					switch stateLoadErr.Cause.(type) {
+					case *state.UndefinedEnvError:
+						return nil
+					default:
+						return ctx.wrapErrs(err)
+					}
 				default:
 					return ctx.wrapErrs(err)
 				}
-			default:
-				return ctx.wrapErrs(err)
 			}
-		}
-		st.Selectors = opts.Selectors
+			st.Selectors = opts.Selectors
 
-		visitSubHelmfiles := func() error {
-			if len(st.Helmfiles) > 0 {
-				noMatchInSubHelmfiles := true
-				for i, m := range st.Helmfiles {
-					optsForNestedState := LoadOpts{
-						CalleePath:        filepath.Join(d, f),
-						Environment:       m.Environment,
-						Reverse:           defOpts.Reverse,
-						RetainValuesFiles: defOpts.RetainValuesFiles,
-					}
-					// assign parent selector to sub helm selector in legacy mode or do not inherit in experimental mode
-					if (m.Selectors == nil && !isExplicitSelectorInheritanceEnabled()) || m.SelectorsInherited {
-						optsForNestedState.Selectors = opts.Selectors
-					} else {
-						optsForNestedState.Selectors = m.Selectors
-					}
+			visitSubHelmfiles := func() error {
+				if len(st.Helmfiles) > 0 {
+					noMatchInSubHelmfiles := true
+					for i, m := range st.Helmfiles {
+						optsForNestedState := LoadOpts{
+							CalleePath:        filepath.Join(d, f),
+							Environment:       m.Environment,
+							Reverse:           defOpts.Reverse,
+							RetainValuesFiles: defOpts.RetainValuesFiles,
+						}
+						if (m.Selectors == nil && !isExplicitSelectorInheritanceEnabled()) || m.SelectorsInherited {
+							optsForNestedState.Selectors = opts.Selectors
+						} else {
+							optsForNestedState.Selectors = m.Selectors
+						}
 
-					if err := a.visitStates(m.Path, optsForNestedState, converge); err != nil {
-						switch err.(type) {
-						case *NoMatchingHelmfileError:
-
-						default:
-							return appError(fmt.Sprintf("in .helmfiles[%d]", i), err)
+						if err := a.visitStates(m.Path, optsForNestedState, converge); err != nil {
+							switch err.(type) {
+							case *NoMatchingHelmfileError:
+							default:
+								return appError(fmt.Sprintf("in .helmfiles[%d]", i), err)
+							}
+						} else {
+							noMatchInSubHelmfiles = false
 						}
-					} else {
-						noMatchInSubHelmfiles = false
 					}
+					noMatchInHelmfiles = noMatchInHelmfiles && noMatchInSubHelmfiles
 				}
-				noMatchInHelmfiles = noMatchInHelmfiles && noMatchInSubHelmfiles
+				return nil
 			}
+
+			if !opts.Reverse {
+				err = visitSubHelmfiles()
+				if err != nil {
+					return err
+				}
+			}
+
+			templated, tmplErr := st.ExecuteTemplates()
+			if tmplErr != nil {
+				return appError(fmt.Sprintf("failed executing release templates in \"%s\"", f), tmplErr)
+			}
+
+			var (
+				processed bool
+				errs      []error
+			)
+
+			CleanWaitGroup.Add(1)
+			defer func() {
+				defer CleanWaitGroup.Done()
+				cleanErr := context{app: a, st: templated, retainValues: defOpts.RetainValuesFiles}.clean(errs)
+				if retErr == nil {
+					retErr = cleanErr
+				} else if cleanErr != nil {
+					a.Logger.Debugf("Failed to clean up temporary files generated while processing %q: %v", templated.FilePath, cleanErr)
+				}
+			}()
+
+			processed, errs = converge(templated)
+
+			noMatchInHelmfiles = noMatchInHelmfiles && !processed
+
+			if opts.Reverse {
+				err = visitSubHelmfiles()
+				if err != nil {
+					return err
+				}
+			}
+
 			return nil
-		}
-
-		if !opts.Reverse {
-			err = visitSubHelmfiles()
-			if err != nil {
-				return err
-			}
-		}
-
-		templated, tmplErr := st.ExecuteTemplates()
-		if tmplErr != nil {
-			return appError(fmt.Sprintf("failed executing release templates in \"%s\"", f), tmplErr)
-		}
-
-		var (
-			processed bool
-			errs      []error
-		)
-
-		// Ensure every temporary files and directories generated while running
-		// the converge function is clean up before exiting this function in all the three cases below:
-		// - This function returned nil
-		// - This function returned an err
-		// - Helmfile received SIGINT or SIGTERM while running this function
-		// For the last case you also need a signal handler in main.go.
-		// Ideally though, this CleanWaitGroup should gone and be replaced by a context cancellation propagation.
-		// See https://github.com/helmfile/helmfile/pull/418 for more details.
-		CleanWaitGroup.Add(1)
-		defer func() {
-			defer CleanWaitGroup.Done()
-			cleanErr := context{app: a, st: templated, retainValues: defOpts.RetainValuesFiles}.clean(errs)
-			if retErr == nil {
-				retErr = cleanErr
-			} else if cleanErr != nil {
-				a.Logger.Debugf("Failed to clean up temporary files generated while processing %q: %v", templated.FilePath, cleanErr)
-			}
-		}()
-
-		processed, errs = converge(templated)
-
-		noMatchInHelmfiles = noMatchInHelmfiles && !processed
-
-		if opts.Reverse {
-			err = visitSubHelmfiles()
-			if err != nil {
-				return err
-			}
-		}
-
-		return nil
-	})
+		})
+	}
 
 	if err != nil {
 		return err
@@ -964,18 +1138,18 @@ var (
 
 func (a *App) ForEachState(do func(*Run) (bool, []error), includeTransitiveNeeds bool, o ...LoadOption) error {
 	ctx := NewContext()
-	err := a.visitStatesWithSelectorsAndRemoteSupport(a.FileOrDir, func(st *state.HelmState) (bool, []error) {
+	err := a.visitStatesWithSelectorsAndRemoteSupportWithContext(a.FileOrDir, func(st *state.HelmState) (bool, []error) {
 		helm, err := a.getHelm(st)
 		if err != nil {
 			return false, []error{err}
 		}
 
-		run, err := NewRun(st, helm, ctx)
+		run, err := NewRun(st, helm, &ctx)
 		if err != nil {
 			return false, []error{err}
 		}
 		return do(run)
-	}, includeTransitiveNeeds, o...)
+	}, includeTransitiveNeeds, &ctx, o...)
 
 	return err
 }
@@ -1021,7 +1195,7 @@ func printDAG(batches [][]state.Release) string {
 
 	_ = w.Flush()
 
-	return buf.String()
+	return trimTrailingWhitespace(buf.String())
 }
 
 // nolint: unparam
@@ -1079,7 +1253,7 @@ type Opts struct {
 	DAGEnabled bool
 }
 
-func (a *App) visitStatesWithSelectorsAndRemoteSupport(fileOrDir string, converge func(*state.HelmState) (bool, []error), includeTransitiveNeeds bool, opt ...LoadOption) error {
+func (a *App) visitStatesWithSelectorsAndRemoteSupportWithContext(fileOrDir string, converge func(*state.HelmState) (bool, []error), includeTransitiveNeeds bool, sharedCtx *Context, opt ...LoadOption) error {
 	opts := LoadOpts{
 		Selectors: a.Selectors,
 	}
@@ -1129,7 +1303,7 @@ func (a *App) visitStatesWithSelectorsAndRemoteSupport(fileOrDir string, converg
 		return f(st)
 	}
 
-	return a.visitStates(fileOrDir, opts, fHelmStatsWithOverrides)
+	return a.visitStatesWithContext(fileOrDir, opts, fHelmStatsWithOverrides, sharedCtx)
 }
 
 func processFilteredReleases(st *state.HelmState, converge func(st *state.HelmState) []error, includeTransitiveNeeds bool) (bool, []error) {
@@ -1384,6 +1558,8 @@ func (a *App) apply(r *Run, c ApplyConfigProvider) (bool, bool, []error) {
 	// helm must be 2.11+ and helm-diff should be provided `--detailed-exitcode` in order for `helmfile apply` to work properly
 	detailedExitCode := true
 
+	detectedKubeVersion := a.detectKubeVersion(st)
+
 	diffOpts := &state.DiffOpts{
 		Color:                   c.Color(),
 		NoColor:                 c.NoColor(),
@@ -1400,6 +1576,7 @@ func (a *App) apply(r *Run, c ApplyConfigProvider) (bool, bool, []error) {
 		SkipSchemaValidation:    c.SkipSchemaValidation(),
 		SuppressOutputLineRegex: c.SuppressOutputLineRegex(),
 		TakeOwnership:           c.TakeOwnership(),
+		DetectedKubeVersion:     detectedKubeVersion,
 	}
 
 	infoMsg, releasesToBeUpdated, releasesToBeDeleted, errs := r.diff(false, detailedExitCode, c, diffOpts)
@@ -1617,6 +1794,29 @@ Do you really want to delete?
 	return true, errs
 }
 
+// detectKubeVersion auto-detects the Kubernetes cluster version if not specified in helmfile.yaml.
+// This prevents helm-diff from falling back to v1.20.0 (issue #2275).
+// Returns empty string when kubeVersion is already set in helmfile.yaml (not needed),
+// when auto-detection is disabled, or if detection fails.
+func (a *App) detectKubeVersion(st *state.HelmState) string {
+	if st.KubeVersion != "" {
+		return ""
+	}
+
+	// Allow tests to disable auto-detection to avoid connecting to real clusters
+	if a.DisableKubeVersionAutoDetection {
+		return ""
+	}
+
+	version, err := cluster.DetectServerVersion(a.Kubeconfig, a.OverrideKubeContext)
+	if err != nil {
+		// If detection fails, we silently continue - helm-diff will handle it
+		return ""
+	}
+
+	return version
+}
+
 func (a *App) diff(r *Run, c DiffConfigProvider) (*string, bool, bool, []error) {
 	var (
 		infoMsg          *string
@@ -1630,6 +1830,8 @@ func (a *App) diff(r *Run, c DiffConfigProvider) (*string, bool, bool, []error)
 
 		helm.SetExtraArgs(GetArgs(c.Args(), r.state)...)
 
+		detectedKubeVersion := a.detectKubeVersion(st)
+
 		opts := &state.DiffOpts{
 			Context:                 c.Context(),
 			Output:                  c.DiffOutput(),
@@ -1645,6 +1847,7 @@ func (a *App) diff(r *Run, c DiffConfigProvider) (*string, bool, bool, []error)
 			SkipSchemaValidation:    c.SkipSchemaValidation(),
 			SuppressOutputLineRegex: c.SuppressOutputLineRegex(),
 			TakeOwnership:           c.TakeOwnership(),
+			DetectedKubeVersion:     detectedKubeVersion,
 		}
 
 		filtered := &Run{
@@ -1982,13 +2185,19 @@ func (a *App) withNeeds(r *Run, c DAGConfig, includeDisabled bool, f func(*state
 		includeNeeds = true
 	}
 
-	batches, err := st.PlanReleases(state.PlanOptions{Reverse: false, SelectedReleases: selectedReleases, IncludeNeeds: includeNeeds, IncludeTransitiveNeeds: c.IncludeTransitiveNeeds(), SkipNeeds: c.SkipNeeds()})
+	batches, err := st.PlanReleases(state.PlanOptions{
+		Reverse:                false,
+		SelectedReleases:       selectedReleases,
+		IncludeNeeds:           includeNeeds,
+		IncludeTransitiveNeeds: c.IncludeTransitiveNeeds(),
+		SkipNeeds:              c.SkipNeeds(),
+	})
+
 	if err != nil {
 		return false, []error{err}
 	}
 
 	var selectedReleasesWithNeeds []state.ReleaseSpec
-
 	for _, rs := range batches {
 		for _, r := range rs {
 			selectedReleasesWithNeeds = append(selectedReleasesWithNeeds, r.ReleaseSpec)
@@ -2023,6 +2232,15 @@ func (a *App) withNeeds(r *Run, c DAGConfig, includeDisabled bool, f func(*state
 		}
 	}
 
+	if c.EnforceNeedsAreInstalled() {
+		for _, r := range toRender {
+			visited := make(map[string]bool)
+			if depErr := st.HasTransitiveDependencyWithInstalledFalse(r, visited); depErr != nil {
+				return false, []error{fmt.Errorf("Release %s has a transitive dependency %s marked as installed=false", depErr.Release.Name, depErr.Dependency.Name)}
+			}
+		}
+	}
+
 	if includeDisabled {
 		for _, d := range releasesToUninstall {
 			rels = append(rels, d)
