@@ -18,6 +18,8 @@ type DiffOptions struct {
 	IncludeNeeds bool
 	// IncludeTransitiveNeeds is the include transitive needs flag
 	IncludeTransitiveNeeds bool
+	// EnforceNeedsAreInstalled is the enforce needs are installed flag
+	EnforceNeedsAreInstalled bool
 	// SkipDiffOnInstall is the skip diff on install flag
 	SkipDiffOnInstall bool
 	// ShowSecrets is the show secrets flag
@@ -87,6 +89,11 @@ func (t *DiffImpl) IncludeTransitiveNeeds() bool {
 	return t.DiffOptions.IncludeTransitiveNeeds
 }
 
+// EnforceNeedsAreInstalled errors if the transitive dependencies are not installable
+func (t *DiffImpl) EnforceNeedsAreInstalled() bool {
+	return t.DiffOptions.EnforceNeedsAreInstalled
+}
+
 // Set returns the Set
 func (t *DiffImpl) Set() []string {
 	return t.DiffOptions.Set
