@@ -30,8 +30,8 @@ func (mock *mockRunner) ExecuteStdIn(cmd string, args []string, env map[string]s
 }
 
 func (mock *mockRunner) Execute(cmd string, args []string, env map[string]string, enableLiveOutput bool) ([]byte, error) {
-	if len(mock.output) == 0 && strings.Join(args, " ") == "version --client --short" {
-		return []byte("v3.2.4+ge29ce2a"), nil
+	if len(mock.output) == 0 && strings.Join(args, " ") == "version --short" {
+		return []byte("v4.0.1+g12500dd"), nil
 	}
 	return mock.output, mock.err
 }
@@ -110,39 +110,45 @@ func Test_SetEnableLiveOutput(t *testing.T) {
 	}
 }
 
-func Test_SetDisableForceUpdate(t *testing.T) {
-	helm, err := MockExecer(NewLogger(os.Stdout, "info"), "config", "dev")
-	if err != nil {
-		t.Error(err)
-	}
-	if helm.options.DisableForceUpdate {
-		t.Error("helmexec.options.ForceUpdate should not be enabled by default")
-	}
-	helm.SetDisableForceUpdate(true)
-	if !helm.options.DisableForceUpdate {
-		t.Errorf("helmexec.SetDisableForceUpdate() - actual = %t expect = true", helm.options.DisableForceUpdate)
-	}
-}
-
 func Test_AddRepo_Helm_Version(t *testing.T) {
 	tests := []struct {
-		name          string
-		version       string
-		disableUpdate bool
-		expected      string
+		name               string
+		version            string
+		disableForceUpdate bool
+		expected           string
 	}{
 		{
-			name:          "Helm 3.3.2 with force update",
-			version:       "3.3.2",
-			disableUpdate: false,
+			name:    "Helm 3.2.0 (before force-update)",
+			version: "3.2.0",
+			expected: `Adding repo myRepo https://repo.example.com/
+exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.example.com/ --cert-file cert.pem --key-file key.pem
+`,
+		},
+		{
+			name:    "Helm 3.3.2 (force-update added)",
+			version: "3.3.2",
 			expected: `Adding repo myRepo https://repo.example.com/
 exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.example.com/ --force-update --cert-file cert.pem --key-file key.pem
 `,
 		},
 		{
-			name:          "Helm 3.3.2 without force update",
-			version:       "3.3.2",
-			disableUpdate: true,
+			name:    "Helm 3.19.2 (with force-update)",
+			version: "3.19.2",
+			expected: `Adding repo myRepo https://repo.example.com/
+exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.example.com/ --force-update --cert-file cert.pem --key-file key.pem
+`,
+		},
+		{
+			name:               "Helm 3.19.2 (force-update disabled)",
+			version:            "3.19.2",
+			disableForceUpdate: true,
+			expected: `Adding repo myRepo https://repo.example.com/
+exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.example.com/ --cert-file cert.pem --key-file key.pem
+`,
+		},
+		{
+			name:    "Helm 4.0.1 (force-update is default)",
+			version: "4.0.1",
 			expected: `Adding repo myRepo https://repo.example.com/
 exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.example.com/ --cert-file cert.pem --key-file key.pem
 `,
@@ -160,7 +166,7 @@ exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.e
 				kubeconfig:  "config",
 				kubeContext: "dev",
 				runner:      &mockRunner{},
-				options:     HelmExecOptions{DisableForceUpdate: tt.disableUpdate},
+				options:     HelmExecOptions{DisableForceUpdate: tt.disableForceUpdate},
 			}
 			err := helm.AddRepo("myRepo", "https://repo.example.com/", "", "cert.pem", "key.pem", "", "", "", false, false)
 
@@ -225,7 +231,7 @@ exec: helm --kubeconfig config --kube-context dev repo add myRepo https://repo.e
 	err = helm.AddRepo("acrRepo", "", "", "", "", "", "", "acr", false, false)
 	expected = `Adding repo acrRepo (acr)
 exec: az acr helm repo add --name acrRepo
-exec: az acr helm repo add --name acrRepo: 
+exec: az acr helm repo add --name acrRepo:
 `
 	if err != nil {
 		t.Errorf("unexpected error: %v", err)
@@ -431,6 +437,7 @@ exec: helm --kubeconfig config --kube-context dev dependency update ./chart/foo
 	buffer.Reset()
 	helm.SetExtraArgs("--verify")
 	err = helm.UpdateDeps("./chart/foo")
+	// --verify is a dependency-specific flag and should be preserved
 	expected = `Updating dependency ./chart/foo
 exec: helm --kubeconfig config --kube-context dev dependency update ./chart/foo --verify
 `
@@ -438,7 +445,7 @@ exec: helm --kubeconfig config --kube-context dev dependency update ./chart/foo
 		t.Errorf("unexpected error: %v", err)
 	}
 	if buffer.String() != expected {
-		t.Errorf("helmexec.AddRepo()\nactual = %v\nexpect = %v", buffer.String(), expected)
+		t.Errorf("helmexec.UpdateDeps()\nactual = %v\nexpect = %v", buffer.String(), expected)
 	}
 }
 
@@ -478,6 +485,7 @@ v3.2.4+ge29ce2a
 	buffer.Reset()
 	helm.SetExtraArgs("--verify")
 	err = helm.BuildDeps("foo", "./chart/foo", []string{"--skip-refresh"}...)
+	// --verify is a dependency-specific flag and should be preserved
 	expected = `Building dependency release=foo, chart=./chart/foo
 exec: helm --kubeconfig config --kube-context dev dependency build ./chart/foo --skip-refresh --verify
 v3.2.4+ge29ce2a
@@ -506,11 +514,52 @@ Client: v2.16.1+ge13bc94
 	if buffer.String() != expected {
 		t.Errorf("helmexec.BuildDeps()\nactual = %v\nexpect = %v", buffer.String(), expected)
 	}
+
+	// Test that --dry-run flag is filtered out (not supported by helm dependency build)
+	buffer.Reset()
+	helm3Runner = mockRunner{output: []byte("v3.2.4+ge29ce2a")}
+	helm, err = New("helm", HelmExecOptions{}, logger, "config", "dev", &helm3Runner)
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	helm.SetExtraArgs("--dry-run=server")
+	err = helm.BuildDeps("foo", "./chart/foo")
+	expected = `Building dependency release=foo, chart=./chart/foo
+exec: helm --kubeconfig config --kube-context dev dependency build ./chart/foo
+v3.2.4+ge29ce2a
+`
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	if buffer.String() != expected {
+		t.Errorf("helmexec.BuildDeps() with --dry-run should filter it out\nactual = %v\nexpect = %v", buffer.String(), expected)
+	}
+
+	// Test that global flags (--debug) and dependency flags (--verify) are preserved,
+	// while template-specific flags (--dry-run) are filtered out
+	buffer.Reset()
+	helm3Runner = mockRunner{output: []byte("v3.2.4+ge29ce2a")}
+	helm, err = New("helm", HelmExecOptions{}, logger, "config", "dev", &helm3Runner)
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	helm.SetExtraArgs("--debug", "--dry-run=server", "--verify", "--wait")
+	err = helm.BuildDeps("foo", "./chart/foo")
+	expected = `Building dependency release=foo, chart=./chart/foo
+exec: helm --kubeconfig config --kube-context dev dependency build ./chart/foo --debug --verify
+v3.2.4+ge29ce2a
+`
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	if buffer.String() != expected {
+		t.Errorf("helmexec.BuildDeps() should preserve global and dependency flags\nactual = %v\nexpect = %v", buffer.String(), expected)
+	}
 }
 
 func Test_DecryptSecret(t *testing.T) {
-	// Set secrets plugin version to 4.0.0
-	if err := os.Setenv("HELM_PLUGINS", "../../test/plugins/secrets/4.0.0"); err != nil {
+	// Set secrets plugin version to 4.7.4
+	if err := os.Setenv("HELM_PLUGINS", "../../test/plugins/secrets/4.7.4"); err != nil {
 		t.Errorf("failed to set environment HELM_PLUGINS error: %s", err)
 	}
 	defer func() {
@@ -561,8 +610,8 @@ Decrypted %s/secretName into %s
 }
 
 func Test_DecryptSecretWithGotmpl(t *testing.T) {
-	// Set secrets plugin version to 4.0.0
-	if err := os.Setenv("HELM_PLUGINS", "../../test/plugins/secrets/4.0.0"); err != nil {
+	// Set secrets plugin version to 4.7.4
+	if err := os.Setenv("HELM_PLUGINS", "../../test/plugins/secrets/4.7.4"); err != nil {
 		t.Errorf("failed to set environment HELM_PLUGINS error: %s", err)
 	}
 	defer func() {
@@ -651,6 +700,140 @@ exec: helm --kubeconfig config --kube-context dev diff upgrade --allow-unrelease
 	}
 }
 
+func Test_DiffRelease_ColorFlagHelm4(t *testing.T) {
+	// Test that --color and --no-color flags are removed and HELM_DIFF_COLOR env var is set for Helm 4
+	var buffer bytes.Buffer
+	logger := NewLogger(&buffer, "debug")
+
+	// MockExecer creates a Helm 4 execer by default (returns v4.0.1)
+	helm, err := MockExecer(logger, "config", "dev")
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+
+	// Verify it's Helm 4
+	if !helm.IsHelm4() {
+		t.Errorf("expected Helm 4, got version: %v", helm.GetVersion())
+	}
+
+	// Test with --color flag
+	buffer.Reset()
+	err = helm.DiffRelease(HelmContext{}, "release", "chart", "default", false, "--color", "--context", "3")
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+
+	// The --color flag should be removed and --context should remain
+	expected := `Comparing release=release, chart=chart, namespace=default
+
+exec: helm --kubeconfig config --kube-context dev diff upgrade --allow-unreleased release chart --context 3
+`
+	actual := buffer.String()
+	if actual != expected {
+		t.Errorf("helmexec.DiffRelease() with --color\nactual = %v\nexpect = %v", actual, expected)
+	}
+
+	// Verify --color flag was removed
+	if strings.Contains(actual, "--color") {
+		t.Errorf("--color flag should have been removed, but got: %v", actual)
+	}
+
+	// Test with --no-color flag
+	buffer.Reset()
+	err = helm.DiffRelease(HelmContext{}, "release", "chart", "default", false, "--no-color", "--context", "3")
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+
+	// The --no-color flag should be removed and --context should remain
+	expected = `Comparing release=release, chart=chart, namespace=default
+
+exec: helm --kubeconfig config --kube-context dev diff upgrade --allow-unreleased release chart --context 3
+`
+	actual = buffer.String()
+	if actual != expected {
+		t.Errorf("helmexec.DiffRelease() with --no-color\nactual = %v\nexpect = %v", actual, expected)
+	}
+
+	// Verify --no-color flag was removed
+	if strings.Contains(actual, "--no-color") {
+		t.Errorf("--no-color flag should have been removed, but got: %v", actual)
+	}
+}
+
+func Test_FilterColorFlagsForHelm4(t *testing.T) {
+	var buffer bytes.Buffer
+	logger := NewLogger(&buffer, "debug")
+	helm, err := MockExecer(logger, "config", "dev")
+	if err != nil {
+		t.Fatalf("unexpected error: %v", err)
+	}
+
+	tests := []struct {
+		name           string
+		inputFlags     []string
+		expectedFlags  []string
+		expectedEnvKey string
+		expectedEnvVal string
+	}{
+		{
+			name:           "color flag",
+			inputFlags:     []string{"--color", "--context", "3"},
+			expectedFlags:  []string{"--context", "3"},
+			expectedEnvKey: "HELM_DIFF_COLOR",
+			expectedEnvVal: "true",
+		},
+		{
+			name:           "no-color flag",
+			inputFlags:     []string{"--no-color", "--context", "3"},
+			expectedFlags:  []string{"--context", "3"},
+			expectedEnvKey: "HELM_DIFF_COLOR",
+			expectedEnvVal: "false",
+		},
+		{
+			name:           "no color flags",
+			inputFlags:     []string{"--context", "3", "--detailed-exitcode"},
+			expectedFlags:  []string{"--context", "3", "--detailed-exitcode"},
+			expectedEnvKey: "",
+			expectedEnvVal: "",
+		},
+		{
+			name:           "color flag with other flags",
+			inputFlags:     []string{"--detailed-exitcode", "--color", "--suppress", "secret"},
+			expectedFlags:  []string{"--detailed-exitcode", "--suppress", "secret"},
+			expectedEnvKey: "HELM_DIFF_COLOR",
+			expectedEnvVal: "true",
+		},
+		{
+			name:           "both color and no-color flags (first wins with defensive check)",
+			inputFlags:     []string{"--color", "--no-color", "--context", "3"},
+			expectedFlags:  []string{"--context", "3"},
+			expectedEnvKey: "HELM_DIFF_COLOR",
+			expectedEnvVal: "true", // Changed: first flag wins due to defensive check
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			env := make(map[string]string)
+			actualFlags := helm.filterColorFlagsForHelm4(tt.inputFlags, env)
+
+			if !reflect.DeepEqual(actualFlags, tt.expectedFlags) {
+				t.Errorf("filterColorFlagsForHelm4() flags\nactual = %v\nexpect = %v", actualFlags, tt.expectedFlags)
+			}
+
+			if tt.expectedEnvKey != "" {
+				if env[tt.expectedEnvKey] != tt.expectedEnvVal {
+					t.Errorf("filterColorFlagsForHelm4() env[%v]\nactual = %v\nexpect = %v",
+						tt.expectedEnvKey, env[tt.expectedEnvKey], tt.expectedEnvVal)
+				}
+			} else if len(env) > 0 {
+				t.Errorf("filterColorFlagsForHelm4() expected no env vars, but got: %v", env)
+			}
+		})
+	}
+}
+
 func Test_DeleteRelease(t *testing.T) {
 	var buffer bytes.Buffer
 	logger := NewLogger(&buffer, "debug")
@@ -1064,30 +1247,50 @@ exec: helm --kubeconfig config --kube-context dev template release https://examp
 }
 
 func Test_IsHelm3(t *testing.T) {
-	helm2Runner := mockRunner{output: []byte("Client: v2.16.0+ge13bc94\n")}
-	helm, err := New("helm", HelmExecOptions{}, NewLogger(os.Stdout, "info"), "", "dev", &helm2Runner)
-	if err != nil {
-		t.Errorf("unexpected error: %v", err)
-	}
-	if helm.IsHelm3() {
-		t.Error("helmexec.IsHelm3() - Detected Helm 3 with Helm 2 version")
-	}
-
 	helm3Runner := mockRunner{output: []byte("v3.0.0+ge29ce2a\n")}
-	helm, err = New("helm", HelmExecOptions{}, NewLogger(os.Stdout, "info"), "", "dev", &helm3Runner)
+	helm, err := New("helm", HelmExecOptions{}, NewLogger(os.Stdout, "info"), "", "dev", &helm3Runner)
 	if err != nil {
 		t.Errorf("unexpected error: %v", err)
 	}
 	if !helm.IsHelm3() {
 		t.Error("helmexec.IsHelm3() - Failed to detect Helm 3")
 	}
+
+	helm4Runner := mockRunner{output: []byte("v4.0.1+g12500dd\n")}
+	helm, err = New("helm", HelmExecOptions{}, NewLogger(os.Stdout, "info"), "", "dev", &helm4Runner)
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	if helm.IsHelm3() {
+		t.Error("helmexec.IsHelm3() - Detected Helm 3 with Helm 4 version")
+	}
+}
+
+func Test_IsHelm4(t *testing.T) {
+	helm3Runner := mockRunner{output: []byte("v3.0.0+ge29ce2a\n")}
+	helm, err := New("helm", HelmExecOptions{}, NewLogger(os.Stdout, "info"), "", "dev", &helm3Runner)
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	if helm.IsHelm4() {
+		t.Error("helmexec.IsHelm4() - Detected Helm 4 with Helm 3 version")
+	}
+
+	helm4Runner := mockRunner{output: []byte("v4.0.1+g12500dd\n")}
+	helm, err = New("helm", HelmExecOptions{}, NewLogger(os.Stdout, "info"), "", "dev", &helm4Runner)
+	if err != nil {
+		t.Errorf("unexpected error: %v", err)
+	}
+	if !helm.IsHelm4() {
+		t.Error("helmexec.IsHelm4() - Failed to detect Helm 4")
+	}
 }
 
 func Test_GetPluginVersion(t *testing.T) {
 	v3ExpectedVersion := "3.15.0"
-	v4ExpectedVersion := "4.0.0"
+	v4ExpectedVersion := "4.7.4"
 	v3PluginDirPath := "../../test/plugins/secrets/3.15.0"
-	v4PluginDirPath := "../../test/plugins/secrets/4.0.0"
+	v4PluginDirPath := "../../test/plugins/secrets/4.7.4"
 
 	v3SecretPluginVersion, err := GetPluginVersion("secrets", v3PluginDirPath)
 	require.NoError(t, err)
@@ -1205,6 +1408,20 @@ func Test_ShowChart(t *testing.T) {
 	}
 }
 
+func Test_SetDisableForceUpdate(t *testing.T) {
+	helm, err := MockExecer(NewLogger(os.Stdout, "info"), "config", "dev")
+	if err != nil {
+		t.Error(err)
+	}
+	if helm.options.DisableForceUpdate {
+		t.Error("helmexec.options.DisableForceUpdate should not be enabled by default")
+	}
+	helm.SetDisableForceUpdate(true)
+	if !helm.options.DisableForceUpdate {
+		t.Errorf("helmexec.SetDisableForceUpdate() - actual = %t expect = true", helm.options.DisableForceUpdate)
+	}
+}
+
 func TestParseHelmVersion(t *testing.T) {
 	tests := []struct {
 		name    string
