@@ -76,7 +76,6 @@ func TestAppendWaitFlags(t *testing.T) {
 		name     string
 		release  *ReleaseSpec
 		syncOpts *SyncOpts
-		helm     helmexec.Interface
 		helmSpec HelmSpec
 		expected []string
 	}{
@@ -85,7 +84,6 @@ func TestAppendWaitFlags(t *testing.T) {
 			name:     "release wait",
 			release:  &ReleaseSpec{Wait: &[]bool{true}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{},
 			expected: []string{"--wait"},
 		},
@@ -93,7 +91,6 @@ func TestAppendWaitFlags(t *testing.T) {
 			name:     "cli flags wait",
 			release:  &ReleaseSpec{},
 			syncOpts: &SyncOpts{Wait: true},
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{},
 			expected: []string{"--wait"},
 		},
@@ -101,7 +98,6 @@ func TestAppendWaitFlags(t *testing.T) {
 			name:     "helm defaults wait",
 			release:  &ReleaseSpec{},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{Wait: true},
 			expected: []string{"--wait"},
 		},
@@ -109,7 +105,6 @@ func TestAppendWaitFlags(t *testing.T) {
 			name:     "release wait false",
 			release:  &ReleaseSpec{Wait: &[]bool{false}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{Wait: true},
 			expected: []string{},
 		},
@@ -117,7 +112,6 @@ func TestAppendWaitFlags(t *testing.T) {
 			name:     "cli flags wait false",
 			release:  &ReleaseSpec{},
 			syncOpts: &SyncOpts{},
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{Wait: true},
 			expected: []string{"--wait"},
 		},
@@ -125,66 +119,58 @@ func TestAppendWaitFlags(t *testing.T) {
 			name:     "helm defaults wait false",
 			release:  &ReleaseSpec{},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{Wait: false},
 			expected: []string{},
 		},
-		// --wait-retries
+		// --wait-retries flag has been removed from Helm
 		{
 			name:     "release wait and retry unsupported",
 			release:  &ReleaseSpec{Wait: &[]bool{true}[0], WaitRetries: &[]int{1}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.11.0"),
 			helmSpec: HelmSpec{},
 			expected: []string{"--wait"},
 		},
 		{
-			name:     "release wait and retry supported",
+			name:     "release wait and retry - retries ignored",
 			release:  &ReleaseSpec{Wait: &[]bool{true}[0], WaitRetries: &[]int{1}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.15.0"),
 			helmSpec: HelmSpec{},
-			expected: []string{"--wait", "--wait-retries", "1"},
+			expected: []string{"--wait"},
 		},
 		{
 			name:     "no wait retry",
 			release:  &ReleaseSpec{WaitRetries: &[]int{1}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.15.0"),
 			helmSpec: HelmSpec{},
 			expected: []string{},
 		},
 		{
-			name:     "cli flags wait and retry",
+			name:     "cli flags wait and retry - retries ignored",
 			release:  &ReleaseSpec{},
 			syncOpts: &SyncOpts{Wait: true, WaitRetries: 2},
-			helm:     testutil.NewVersionHelmExec("3.15.0"),
 			helmSpec: HelmSpec{},
-			expected: []string{"--wait", "--wait-retries", "2"},
+			expected: []string{"--wait"},
 		},
 		{
-			name:     "helm defaults wait retry",
+			name:     "helm defaults wait retry - retries ignored",
 			release:  &ReleaseSpec{},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.15.0"),
 			helmSpec: HelmSpec{Wait: true, WaitRetries: 3},
-			expected: []string{"--wait", "--wait-retries", "3"},
+			expected: []string{"--wait"},
 		},
 		{
-			name:     "release wait default retries",
+			name:     "release wait default retries - retries ignored",
 			release:  &ReleaseSpec{Wait: &[]bool{true}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.15.0"),
 			helmSpec: HelmSpec{WaitRetries: 4},
-			expected: []string{"--wait", "--wait-retries", "4"},
+			expected: []string{"--wait"},
 		},
 		{
-			name:     "release retries default wait",
+			name:     "release retries default wait - retries ignored",
 			release:  &ReleaseSpec{WaitRetries: &[]int{5}[0]},
 			syncOpts: nil,
-			helm:     testutil.NewVersionHelmExec("3.15.0"),
 			helmSpec: HelmSpec{Wait: true},
-			expected: []string{"--wait", "--wait-retries", "5"},
+			expected: []string{"--wait"},
 		},
 	}
 
@@ -192,7 +178,7 @@ func TestAppendWaitFlags(t *testing.T) {
 		t.Run(tt.name, func(t *testing.T) {
 			st := &HelmState{}
 			st.HelmDefaults = tt.helmSpec
-			got := st.appendWaitFlags([]string{}, tt.helm, tt.release, tt.syncOpts)
+			got := st.appendWaitFlags([]string{}, tt.release, tt.syncOpts)
 			require.Equalf(t, tt.expected, got, "appendWaitFlags() = %v, want %v", got, tt.expected)
 		})
 	}
