@@ -13,9 +13,9 @@ func testYamlMarshal(t *testing.T, GoYamlV3 bool) {
 
 	var yamlLibraryName string
 	if GoYamlV3 {
-		yamlLibraryName = "gopkg.in/yaml.v3"
+		yamlLibraryName = "go.yaml.in/yaml/v3"
 	} else {
-		yamlLibraryName = "gopkg.in/yaml.v2"
+		yamlLibraryName = "go.yaml.in/yaml/v2"
 	}
 
 	v := runtime.GoYamlV3
@@ -49,8 +49,8 @@ func testYamlMarshal(t *testing.T, GoYamlV3 bool) {
 				Annotation: "on",
 			}},
 			expected: map[string]string{
-				"gopkg.in/yaml.v2": "name: John\ninfo:\n- age: 20\n  address: New York\n  annotation: \"on\"\n",
-				"gopkg.in/yaml.v3": "name: John\ninfo:\n  - age: 20\n    address: New York\n    annotation: \"on\"\n",
+				"go.yaml.in/yaml/v2": "name: John\ninfo:\n- age: 20\n  address: New York\n  annotation: \"on\"\n",
+				"go.yaml.in/yaml/v3": "name: John\ninfo:\n  - age: 20\n    address: New York\n    annotation: \"on\"\n",
 			},
 		},
 	}
@@ -63,11 +63,11 @@ func testYamlMarshal(t *testing.T, GoYamlV3 bool) {
 }
 
 func TestYamlMarshal(t *testing.T) {
-	t.Run("with gopkg.in/yaml.v2", func(t *testing.T) {
+	t.Run("with go.yaml.in/yaml/v2", func(t *testing.T) {
 		testYamlMarshal(t, true)
 	})
 
-	t.Run("with gopkg.in/yaml.v3", func(t *testing.T) {
+	t.Run("with go.yaml.in/yaml/v3", func(t *testing.T) {
 		testYamlMarshal(t, false)
 	})
 }
